﻿/*
 * Ejection Seat
 * Author: Pandassaurus
 * Version: 0.1
 * Donate if you enjoy!
*/

using GTA;
using System;
using System.Windows.Forms;
using GTA.Native;
using GTA.Math;

public static class UTILS
{
    //Thanks to libertylocked for this, as well as the prop and teleport functions!
    //Check out his fishing mod: https://www.gta5-mods.com/scripts/fishing-mod-libertylocked
    public static int PedBoneIndex(this Ped p, int b)
    {
        return Function.Call<int>(Hash.GET_PED_BONE_INDEX, p.Handle, b);
    }
}

public class EjectionSeat : Script
{
    //TODO: Controller Support, Customizable Keys, Better Launching(way you're facing, not just up), some toggleable options(such as countdown, fire trail, realism), add sounds


    private bool uCountdown = false; //default false
    private float uPushForce = 8; //default 8
    private int uTime = 156; //default 156

    private bool realism = true; //default true
    private String version = "v0.1";
    private bool hasBeenPlugged = false;


    bool giveParachute = false;
    bool blowUpVehicle = false;
    private Prop rocket = null;

    public EjectionSeat()
    {
        Tick += OnTick;
        KeyUp += OnKeyUp;
    }

    void OnTick(object sender, EventArgs e)
    {
        Ped playerPed = Game.Player.Character;     
 
        //When character is falling from launch
        if (Function.Call<bool>(Hash.IS_PED_FALLING, playerPed) && giveParachute)
        {
            //Stop Animation, give parachute, delete rocket
            playerPed.Task.ClearAllImmediately();
            playerPed.Task.PutAwayParachute();
            rocket.Delete();
            giveParachute = false;
            if (blowUpVehicle) UI.ShowSubtitle("");
            blowUpVehicle = false;

            //Remove entity proofs
            Function.Call(Hash.SET_ENTITY_PROOFS, playerPed, false, false, false, false, false, false, false, false);
        }       
    }

    void OnKeyUp(object sender, KeyEventArgs e)
    {
        Ped pedPlayer = Game.Player.Character;

        if (e.KeyCode == Keys.Enter)
        {
            if (pedPlayer.IsInVehicle() && !Function.Call<bool>(Hash.IS_PAUSE_MENU_ACTIVE))
            {
                //Name of the mod
                if (Game.Player.IsPlaying && !hasBeenPlugged)
                {
                    GTA.UI.Notify("Ejection Seat " + version + "\nBy Pandassaurus");
                    hasBeenPlugged = true;
                }

                Vehicle vehicle = pedPlayer.CurrentVehicle;
                String vDName = vehicle.DisplayName;

                //Convertibles
                if (vehicle.IsConvertible)
                {
                    //Open Roof
                    //Function.Call(Hash.SET_ENTITY_ANIM_SPEED, vehicle)
                    vehicle.RoofState = VehicleRoofState.Opened;
                    GTA.UI.ShowSubtitle("Initiating Sequence...", 100000);

                    //Wait to open
                    while (vehicle.RoofState != VehicleRoofState.Opened)
                    {
                        vehicle.RoofState = VehicleRoofState.Opened;
                        if (!pedPlayer.IsSittingInVehicle())
                        {
                            GTA.UI.ShowSubtitle("Sequence Aborted");
                            return;
                        }
                        Script.Wait(10);
                    }

                    //Launch!
                    //veh@plane@lazer@exit_skydive -> fds_jump_out_skydive
                    if (!EjectPlayer(vehicle, "amb@code_human_in_car_idles@generic@ds@idle_b", "idle_d"))
                    {
                        GTA.UI.ShowSubtitle("Sequence Aborted.");
                    }
                }

                //Planes
                else if (vDName == "DUSTER" || vDName == "BESRA" || vDName == "HYDRA" || vDName == "LAZER" ||
                         vDName == "STUNT")
                {
                    if (!EjectPlayer(vehicle, "amb@code_human_in_car_idles@generic@ds@idle_b", "idle_d"))
                    {
                        GTA.UI.ShowSubtitle("Sequence Aborted.");
                    }
                }

                //Topless Cars
                else if (!vehicle.HasRoof)
                {
                    if (!EjectPlayer(vehicle, "amb@code_human_in_car_idles@generic@ds@idle_b", "idle_d"))
                    {
                        GTA.UI.ShowSubtitle("Sequence Aborted.");
                    }
                }

                //Bikes and motorbikes and quadbikes
                else if (vehicle.Model.IsBike || vehicle.Model.IsBicycle || vehicle.Model.IsQuadbike)
                {
                    if (!EjectPlayer(vehicle, "veh@bike@chopper@front@base", "lean_r_slow_extreme"))
                    {
                        GTA.UI.ShowSubtitle("Sequence Aborted.");
                    }
                }

                //Incompatable cars
                else
                {
                    if (realism)
                    {
                        GTA.UI.ShowSubtitle("You must be in a vehicle without a roof.");
                    }
                    else
                    {
                        if (!EjectPlayer(vehicle, "veh@bike@chopper@front@base", "lean_r_slow_extreme"))
                        {
                            GTA.UI.ShowSubtitle("Sequence Aborted.");
                        }
                    }
                }
            }
        }


        //Explode Vehicle
        if (e.KeyCode == Keys.E && blowUpVehicle)
        {           
            World.AddExplosion(Game.Player.LastVehicle.Position, ExplosionType.ExplosionWithFire1, 200, 0.4f);
            Game.Player.LastVehicle.Explode();
            UI.ShowSubtitle("");
            blowUpVehicle = false;
        }
    }

    bool EjectPlayer(Vehicle vehicle, String animSet, String animName)
    {
        Ped playerPed = Game.Player.Character;
        //CountDown or no countdown, it's an option
        if (uCountdown)
        {
            GTA.UI.ShowSubtitle("Launching in 3...", 500);
            for (int x = 0; x < 25; x++)
            {
                if (!playerPed.IsSittingInVehicle()) return false;
                Script.Wait(20);
            }
            GTA.UI.ShowSubtitle("Launching in 2...", 500);
            for (int x = 0; x < 25; x++)
            {
                if (!playerPed.IsSittingInVehicle()) return false;
                Script.Wait(20);
            }
            GTA.UI.ShowSubtitle("Launching in 1...", 500);
            for (int x = 0; x < 25; x++)
            {
                if (!playerPed.IsSittingInVehicle() && x < 22) return false;
                if (x == 23 && vehicle.Model.IsPlane) vehicle.OpenDoor(VehicleDoor.FrontLeftDoor, false, true);
                Script.Wait(20);
            }
            GTA.UI.ShowSubtitle("Enjoy your flight", 300);
        }
        else
        {
            if (vehicle.Model.IsPlane) vehicle.OpenDoor(VehicleDoor.FrontLeftDoor, false, true);
            Script.Wait(20);
        }


        //Teleport, set up blow up car       
        Vector3 playerInCarCoords = Function.Call<Vector3>(Hash.GET_ENTITY_COORDS, playerPed.Handle, true);
        Function.Call(Hash.SET_ENTITY_COORDS, playerPed.Handle, playerInCarCoords.X + playerPed.UpVector.X, playerInCarCoords.Y + playerPed.UpVector.Y, 
            playerInCarCoords.Z + playerPed.UpVector.Z, true, true, true, 1);
        blowUpVehicle = true;
        GTA.UI.ShowSubtitle("Press \"E\" to blow up your vehicle.", 10000);
        
        //Let player live through this
        playerPed.CanRagdoll = false;
        Function.Call(Hash.SET_ENTITY_PROOFS, playerPed, false, true, true, false, false, false, false, false);
       
        //Animation and attatch rocket   
        playerPed.Task.PlayAnimation(animSet, animName, 24f, -1, true, 3f);
        if(rocket != null) rocket.Delete();
        rocket = World.CreateProp(new Model("w_lr_homing_rocket"), Vector3.Zero, false, false);
        rocket.AttachTo(playerPed, playerPed.PedBoneIndex(Bone.SKEL_Pelvis.GetHashCode()), new Vector3(0.25f, -0.05f, 0f), new Vector3(0f, 0f, 70f));   

        //Gradual Launch with FIRE, then allow ragdoll and give parachute
        for (int i = 0; i < uTime; i++)
        {
            if (!Function.Call<bool>(Hash.IS_PED_FALLING, playerPed))
            {
                if (i == 2 || i == 6 || i == 14 || i == 24 || i == 36 || i == 52 || i == 72 || i == 96 || i == 124 ||
                    i == 156)
                {
                    playerPed.ApplyForce(new Vector3(0f, 0f, uPushForce));
                }
                if(i > 10) World.AddExplosion(rocket.Position + new Vector3(0, 0, 1f), ExplosionType.ValveFire2, 2.5f, 6f);
                Script.Wait(1);
            }
            else
            {
                break;
            }
        }
        playerPed.CanRagdoll = true;
        giveParachute = true;
        return true;
    }
}